require 'fileutils'
require 'json'
require 'salesforce_fsdb/rest_client'

module SalesforceFsdb
  class Client
    attr_accessor :oSfRestClient

    def initialize(dParamsGeneral={},dParamsToken={})
      @oSfRestClient  = SalesforceFsdb::RestClient.new(dParamsGeneral,dParamsToken)
      @sFsdbBaseDir   = dParamsGeneral[:data_dir] || '.'
      @iMaxAgeSec     = dParamsGeneral[:max_age]  || 3600*24*7
    end

    def getFileForSfidAndType(sSfid=nil,sType=nil)
      sFile  = %Q{sf_#{sType}_#{sSfid}.json}
      return sFile
    end

    def getDirForType(sType=nil)
      sDir   = File.join(@sFsdbBaseDir,sType)
      return sDir
    end

    def getPathForSfidAndType(sSfid=nil,sType=nil)
      sPath  = File.join( self.getDirForType(sType), self.getFileForSfidAndType(sSfid,sType))
      return sPath
    end

    def getResForSoqlAndPath(sSoql=nil,sPath=nil)
      return nil if sSoql.nil?
      if ! sPath.nil? && sPath !~ /^\s*\//
        sPath.strip!
        sPath = File.join(@sFsdbBaseDir,sPath)
      end
      dRes      = { :meta => { :iEpochRetrieved => -1, :iStatus => 404 } }
      if !sPath.nil? && File.exists?(sPath)
        jTop    = File.open(sPath,'r').read
        dTop    = JSON.parse(jTop,:symbolize_names=>true)
        if dTop.has_key?(:source) && dTop.has_key?(:meta) && dTop[:meta].has_key?(:iEpochRetrieved)
          iEpochRetrieved = dTop[:meta][:iEpochRetrieved]
          iEpochNow       = Time.now.to_i
          iAgeSec         = iEpochNow - iEpochRetrieved
          if iAgeSec < @iMaxAgeSec && dTop[:source].has_key?(:done)
            dRes = dTop[:source]
            return dRes
          end
        end
      end
      return self.getResForSoqlAndPathFromRemote(sSoql,sPath)
    end

    def getResForSoqlAndPathFromRemote(sSoql=nil,sPath=nil)
      return nil if sSoql.nil?
      if ! sPath.nil? && sPath !~ /^\s*\//
        sPath.strip!
        sPath = File.join(@sFsdbBaseDir,sPath)
      end
      dRes = @oSfRestClient.getSoqlResults(sSoql)
      if ! dRes.nil? && dRes.has_key?(:done) && dRes[:done] == true && ! sPath.nil?
        if sPath =~ /^([\S]+)\/[^\/]+$/
          sDir   = $1
          FileUtils.mkdir_p(sDir) if ! Dir.exists?(sDir)
        end
        File.open(sPath,'w') do |fTop|
          dTop      =  {
            :meta   => { :iEpochRetrieved => Time.now.to_i, :iStatus => 200 },
            :source => dRes
          }
          jTop = JSON.dump(dTop)
          fTop.puts(jTop)
        end
        return dRes
      end
      return nil
    end

    def getSobjectForSfidAndType(sSfid=nil,sType=nil)
      sPath     = self.getPathForSfidAndType(sSfid,sType)
      dObj      = { :meta => { :iEpochRetrieved => -1, :iStatus => 404 } }
      if File.exists?(sPath)
        jTop    = File.open(sPath,'r').read
        dTop    = JSON.parse(jTop,:symbolize_names=>true)
        if dTop.has_key?(:source) && dTop.has_key?(:meta) && dTop[:meta].has_key?(:iEpochRetrieved)
          iEpochRetrieved = dTop[:meta][:iEpochRetrieved]
          iEpochNow       = Time.now.to_i
          iAgeSec         = iEpochNow - iEpochRetrieved
          if iAgeSec < @iMaxAgeSec && dTop[:source].has_key?(:Id)
            dObj = dTop[:source]
            return dObj
          end
        end
      end
      return self.getSobjectForSfidAndTypeFromRemote(sSfid,sType)
    end

    def getSobjectForSfidAndTypeFromRemote(sSfid=nil,sType=nil)
      dObj = @oSfRestClient.getSobjectForSfidAndType(sSfid,sType)
      if (dObj.is_a?(Array) && dObj[0][:errorCode] == 'NOT_FOUND') || dObj.nil?
        if sType == 'Account'

          dObjOppTry = self.getSobjectForSfidAndType(sSfid,'Opportunity')

          if dObjOppTry.is_a?(Hash) && dObjOppTry.has_key?(:AccountId)
            sSfidAct = dObjOppTry[:AccountId]
            return self.getSobjectForSfidAndType(sSfidAct,sType)
          end

        else
          return nil
        end
      end
      if dObj.is_a?(Hash) && dObj.has_key?(:Id)
        sDir        = self.getDirForType(sType)
        sPath       = self.getPathForSfidAndType(sSfid,sType)
        FileUtils.mkdir_p(sDir) if ! Dir.exists?(sDir)
        File.open(sPath,'w') do |fTop|
          dTop      =  {
            :meta   => { :iEpochRetrieved => Time.now.to_i, :iStatus => 200 },
            :source => dObj
          }
          jTop = JSON.dump(dTop)
          fTop.puts(jTop)
        end
        return dObj
      end
      return nil
    end

  end
end