require 'faraday'
require 'json'

module SalesforceFsdb
  class RestClient

    attr_accessor :sSfTokenUrl
    attr_accessor :dTokenRes

    def initialize(dParamsGeneral={},dParamsToken={})
      @dParamsToken = dParamsToken
      @sSfTokenUrl  = dParamsGeneral.has_key?(:token_url)   ? dParamsGeneral[:token_url] \
                    : 'https://login.salesforce.com/services/oauth2/token'
      @sApiFqdn     = dParamsGeneral.has_key?(:api_fqdn)    ? dParamsGeneral[:api_fqdn] : 'na5.salesforce.com'
      @sApiVersion  = dParamsGeneral.has_key?(:api_version) ? dParamsGeneral[:api_version] : '27.0'
      @dTokenRes    = {}
      self.loadToken()
    end

    def loadToken()
      oRes       = Faraday.post(@sSfTokenUrl,@dParamsToken) 
      @dTokenRes = JSON.parse(oRes.body,:symbolize_names=>true)
      return self
    end

    def getAttr(yKey=nil)
      yKey  = yKey.to_sym if yKey.is_a?(String)
      xxVal = @dTokenRes.has_key?(yKey) ? @dTokenRes[yKey] : nil
      return xxVal
    end

    def getHeaders()
      dHeaders = { 'Authorization' => 'Bearer ' + self.getAttr(:access_token) }
      return dHeaders
    end

    def getSoqlResults(sSoql=nil)
      dQry  = { :q => sSoql}
      sUrl  = "https://#{@sApiFqdn}/services/data/v#{@sApiVersion}/query"
      dHead = self.getHeaders()
      oRes  = Faraday.get(sUrl,dQry,dHead)
      dRes  = JSON.parse(oRes.body,:symbolize_names=>true)
      return dRes
    end

    def getSfUrlForSfidAndType(sSfid=nil,sType=nil)
      sUrl  = "https://#{@sApiFqdn}/services/data/v#{@sApiVersion}/sobjects/#{sType}/#{sSfid}/"
      return sUrl
    end

    def getSobjectForSfidAndType(sSfid=nil,sType=nil)
      sUrl  = self.getSfUrlForSfidAndType(sSfid,sType)
      sHead = self.getHeaders()
      oRes  = Faraday.get(sUrl,{},sHead)
      dRes  = JSON.parse(oRes.body,:symbolize_names=>true)
      return dRes
    end

  end
end